### KOOPERATIONS NETZWERKE ###


#### load data ----

# load preprocessed PSYNDEX Export
load("klinpsy_KP.RData") 
# data frame with abstracts (ABH), titles (TI), translated titles (TIU), controlled terms (CTd), 
# uncontrolled terms (UTG), additional terms (ITd), author information (YYKLAU), publication year (PY)




#### text mine the author field ----

# https://stackoverflow.com/questions/5318076/extracting-country-name-from-author-affiliations
library(maps)
# Load data from package maps
data(world.cities)
countries <- tolower(unique(world.cities$country.etc))
countries <- c(countries, "united kingdom", "united states", "peoples republic of china")
countries <- gsub(" ", "_", countries)

# Countries are capitalized in YYKLAU
# https://stackoverflow.com/questions/36547412/keeping-only-fully-capitalized-words-in-string-using-r
library(stringr)
klinpsy_KP$countries <- sapply(str_extract_all(klinpsy_KP$YYKLAU, "\\b[A-Z]+\\b"), paste, collapse = ' ')
klinpsy_KP$countries <- tolower(klinpsy_KP$countries)
# instiution names that are capitalized still remain (e.g., ZPID)

# the following list is created iteratively by checking DFM topfeatures and countries list
klinpsy_KP$countries <- gsub("united states", "united_states", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("united kingdom", "united_kingdom", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("bosnia and herzegovina", "bosnia_and_herzegovina", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("peoples republic of china", "peoples_republic_of_china", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("new zealand", "new_zealand", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("czech republic", "czech_republic", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("south africa", "south_africa", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("serbia and montenegro", "serbia_and_montenegro", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("south korea", "korea_south", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("north korea", "korea_north", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("burkina faso", "burkina_faso", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("sri lanka", "sri_lanka", klinpsy_KP$countries)
klinpsy_KP$countries <- gsub("central african republic", "central_african_republic", klinpsy_KP$countries)
# remove "usa" (gets double counted)
klinpsy_KP$countries <- gsub("usa", "", klinpsy_KP$countries)

# https://stackoverflow.com/questions/33226616/how-to-remove-words-of-specific-length-in-a-string-in-r
klinpsy_KP$countries <- gsub('\\b\\w{1,2}\\b','', klinpsy_KP$countries) # remove words with length 2


library(quanteda)
country_corpus <- corpus(klinpsy_KP$countries)

country_DFM <- country_corpus %>% 
  tokens() %>% 
  tokens_remove(pattern = c(stopwords("en"), stopwords("de"))) %>%
  tokens(remove_punct = TRUE, remove_numbers = TRUE, remove_symbols = TRUE) %>% dfm()

topfeatures(country_DFM, n = 100)  



#### number of countries per doc ----

# keep only countries in DFM
# https://datascience.stackexchange.com/questions/8519/extracting-list-of-locations-from-text-using-r

temp <- colnames(country_DFM)
# Return the index (position) of matching elements
x <- which(temp %in% countries) 

# DFM with countries only
countries_DFM <- country_DFM[, x]
topfeatures(countries_DFM, n = 20) 

# see which documents have multiple country information
table(rowSums(countries_DFM))

# how many countries per doc?
klinpsy_KP$Ncountries <- rowSums(countries_DFM)



## use only unique countries per doc
# e.g., three autors GER GER USA -> GER-USA 1 (no GER-GER)
unique <- countries_DFM
unique[unique > 1] <- 1
unique <- as.dfm(unique)

# how many different countries per doc?
table(rowSums(unique))
# check example
klinpsy_KP[which(rowSums(unique) == 11),]


## feature co-occurance matrix
unique_FCM <- fcm(unique, tri = FALSE)

germany_collabos <- convert(unique_FCM[, which(colnames(unique_FCM) == "germany")], to = "data.frame")
colnames(germany_collabos) <- c("Land", "Anzahl Kooperationen")
germany_collabos <- germany_collabos[order(-germany_collabos$`Anzahl Kooperationen`),]

austria_collabos <- convert(unique_FCM[, which(colnames(unique_FCM) == "austria")], to = "data.frame")
colnames(austria_collabos) <- c("Land", "Anzahl Kooperationen")
austria_collabos <- austria_collabos[order(-austria_collabos$`Anzahl Kooperationen`),]

switzerland_collabos <- convert(unique_FCM[, which(colnames(unique_FCM) == "switzerland")], to = "data.frame")
colnames(switzerland_collabos) <- c("Land", "Anzahl Kooperationen")
switzerland_collabos <- switzerland_collabos[order(-switzerland_collabos$`Anzahl Kooperationen`),]

## network plot
set.seed(2811285)
textplot_network(unique_FCM)


write.csv2(germany_collabos, file = "germany_collabos.csv", row.names = FALSE)