# original code by A. Niekler & G. Wiedemann

# Functions
topicProbability <- function(topics) {
 token <- sum(topics)
 prob <- c()
 for (i in 1:nrow(topics)) {
  ts <- sum(topics[i, ]) / token
  prob <- c(prob, ts)
 }
 return(prob)
}

toComparable <- function(topics, topWordsToMatch) {
 for (i in 1:nrow(topics)) {
  ts <- topics[i, ] / sum(topics[i, ])
  left <- names(ts[order(-ts)][1:topWordsToMatch])
  topics[i, ] <- 0
  topics[i, left] <- ts[left]
 }
 return(topics)
}

fromLDAPackage <- function(tm) {
 convertedTM <- list()
 #convertedTM$p_t_d <- tm$assignments
 convertedTM$p_w_t <- toComparable(tm$topics)
 convertedTM$p_t <-
  colSums(convertedTM$p_t_d) / sum(convertedTM$p_t_d)
}

fromTopicModelsPackage <- function(tm) {
 convertedTM <- list()
 post <- posterior(tm)
 convertedTM$p_t <- colSums(post$topics) / sum(post$topics)
 convertedTM$p_t_d <- post$topics
 convertedTM$p_w_t <- post$terms
 return(convertedTM)
}

# Lancichinetti
get_p_d_t <- function(tm)
{
 p_d_t <- t(tm$p_t_d) / (tm$p_t * nrow(tm$p_t_d))
 return(p_d_t)
}

TM_Aligner <- function(tm1, 
            tm2, 
            thres = 0, 
            method = "jenson", 
            topWordsToMatch = NULL, # integer
            validWords = NULL, # character vector
            calculateBM = F) {
 K <- nrow(tm1$p_w_t)
 reliability <- 0
 similarity <- 0
 
 alignedTopics <- list()
 alignedTopics$ids <- matrix(0, nrow = K, ncol = 2)
 alignedTopics$probabilities <- matrix(0, nrow = K, ncol = 2)
 alignedTopics$sharedTerms <- vector("list", K)
 alignedTopics$similarity <- rep(0, K)
 
 wordIds <- 1:ncol(tm1$p_w_t)
 
 if (!is.null(validWords))
 {
  wordIds <- intersect(wordIds, validWords)
 }
 
 if (!is.null(topWordsToMatch))
 {
  tm1$p_w_t <- t(apply(tm1$p_w_t, 1, function(x) {
   x[order(x, decreasing = T)[(topWordsToMatch + 1):length(x)]] <- 0.
   return(x)
  }))
  
  tm2$p_w_t <- t(apply(tm2$p_w_t, 1, function(x) {
   x[order(x, decreasing = T)[(topWordsToMatch + 1):length(x)]] <- 0.
   return(x)
  }))
 }
 else
 {
  # just for display of matching terms
  topWordsToMatch <- 40
 }
 
 # for simil function
 require(proxy)
 
 switch (
  method, 
  cosine = {
   similarity <-
    simil(
     tm1$p_w_t[, wordIds], tm2$p_w_t[, wordIds], method = "cosine", pairwise = F, by_rows = T
    )
   similarity2 <-
    simil(
     tm2$p_w_t[, wordIds], tm1$p_w_t[, wordIds], method = "cosine", pairwise = F, by_rows = T
    )
   similarity_rand <-
    simil(
     tm1$p_w_t[, wordIds], tm2$p_w_t[, wordIds][, sample.int(dim(tm2$p_w_t[,  wordIds])[2])], method =
      "cosine", pairwise = F, by_rows = T
    )
   similarity2_rand <-
    simil(
     tm2$p_w_t[, wordIds], tm1$p_w_t[, wordIds][, sample.int(dim(tm1$p_w_t[, wordIds])[2])], method =
      "cosine", pairwise = F, by_rows = T
    )
   
  }, 
  jensen = {
   f <- function(x, y) {
    m <- 0.5 * (x + y)
    JS <-
     0.5 * (sum(x * log(x / m)) + sum(y * log(y / m)))
    return(JS)
   }
   
   similarity <-
    1 - dist(tm1$p_w_t[, wordIds], tm2$p_w_t[, wordIds], f, pairwise = F, by_rows = T)
   similarity2 <-
    1 - dist(tm2$p_w_t[, wordIds], tm1$p_w_t[, wordIds], f, pairwise = F, by_rows = T)
   similarity_rand <-
    1 - dist(tm1$p_w_t[, wordIds], tm2$p_w_t[, wordIds][, sample.int(dim(tm2$p_w_t[, wordIds])[2])], f, pairwise = F, by_rows = T)
   similarity2_rand <-
    1 - dist(tm2$p_w_t[, wordIds], tm1$p_w_t[, wordIds][, sample.int(dim(tm1$p_w_t[, wordIds])[2])], f, pairwise = F, by_rows = T)
  }, 
  kullback = {
   similarity <-
    simil(
     tm1$p_w_t[, wordIds], tm2$p_w_t[, wordIds], method = "Kullback", pairwise = F, by_rows = T
    )
   similarity2 <-
    simil(
     tm2$p_w_t[, wordIds], tm1$p_w_t[, wordIds], method = "Kullback", pairwise = F, by_rows = T
    )
   similarity_rand <-
    simil(
     tm1$p_w_t[, wordIds], tm2$p_w_t[, wordIds][, sample.int(dim(tm2$p_w_t[, wordIds])[2])], method =
      "Kullback", pairwise = F, by_rows = T
    )
   similarity2_rand <-
    simil(
     tm2$p_w_t[, wordIds], tm1$p_w_t[, wordIds][, sample.int(dim(tm1$p_w_t[, wordIds])[2])], method =
      "Kullback", pairwise = F, by_rows = T
    )
  }, 
  doctopic = {
   # manhatten distance
   f <- function(x, y)
    1 - 0.5 * sum(abs(x - y))
   similarity <-
    simil(get_p_d_t(tm1), get_p_d_t(tm2), f, pairwise = F, by_rows = T)
   similarity2 <-
    simil(get_p_d_t(tm2), get_p_d_t(tm1), f, pairwise = F, by_rows = T)
   similarity_rand <-
    simil(get_p_d_t(tm1), get_p_d_t(tm2)[, sample.int(ncol(get_p_d_t(tm2)))], f, pairwise = F, by_rows = T)
   similarity2_rand <-
    simil(get_p_d_t(tm2), get_p_d_t(tm1)[, sample.int(ncol(get_p_d_t(tm1)))], f, pairwise = F, by_rows = T)
  },
  {
    stop(paste0('method "', method,'" not implemented'))
  }
 )
 
 idx <- apply(similarity, MARGIN = 1, which.max)
 idx2 <- apply(similarity2, MARGIN = 1, which.max)
 idx_rand <- apply(similarity_rand, MARGIN = 1, which.max)
 idx2_rand <- apply(similarity2_rand, MARGIN = 1, which.max)
 
 similarity_orig <- similarity
 similarity_orig2 <- similarity2
 similarity_rand_orig <- similarity_rand
 similarity_rand_orig2 <- similarity2_rand
 
 for (i in 1:K) {
  index <- arrayInd(which.max(similarity), .dim = c(K, K))
  index2 <- arrayInd(which.max(similarity2), .dim = c(K, K))
  
  index_rand <-
   arrayInd(which.max(similarity_rand), .dim = c(K, K))
  index_rand2 <-
   arrayInd(which.max(similarity2_rand), .dim = c(K, K))
  
  idx[index[1]] <- index[2]
  idx2[index2[1]] <- index2[2]
  
  idx_rand[index_rand[1]] <- index_rand[2]
  idx2_rand[index_rand2[1]] <- index_rand2[2]
  
  
  value <- max(similarity)
  
  similarity[index[1], ] <- 0
  similarity[, index[2]] <- 0
  
  similarity2[index2[1], ] <- 0
  similarity2[, index2[2]] <- 0
  
  similarity_rand[index_rand[1], ] <- 0
  similarity_rand[, index_rand[2]] <- 0
  
  similarity2_rand[index_rand2[1], ] <- 0
  similarity2_rand[, index_rand2[2]] <- 0
  
  # thres > 0 does not make sense for doctopic approach
  if (value < thres)
   break
  
  alignedTopics$ids[i, ] <- c(index[1], index[2])
  alignedTopics$probabilities[i, ] <- c(tm1$p_t[index[1]],  tm2$p_t[index[2]])
  alignedTopics$sharedTerms[[i]] <- intersect(
    names(tm1$p_w_t[index[1], ][order(-tm1$p_w_t[index[1], ])][1:topWordsToMatch]), 
    names(tm2$p_w_t[index[2], ][order(-tm2$p_w_t[index[2], ])][1:topWordsToMatch]))
  
  # count matched topics
  reliability = reliability + 1
  
  alignedTopics$similarity[i] <- value
  
 }
 
 if (calculateBM)
 {
  reliability <- 0.5 * (sum(tm1$p_t[1:length(idx)] * similarity_orig[cbind(1:length(idx), idx)]) + sum(tm2$p_t[1:length(idx2)] * similarity_orig2[cbind(1:length(idx2), idx2)]))
  #Zufällig gezogen als Vergleich
  reliability_rand <-
   0.5 * (sum(tm1$p_t[1:length(idx_rand)] * similarity_rand_orig[cbind(1:length(idx_rand), idx_rand)]) + sum(tm2$p_t[1:length(idx2_rand)] * similarity_rand_orig2[cbind(1:length(idx2_rand), idx2_rand)]))
  reliability <- (reliability - reliability_rand) / (1 - reliability_rand)
  alignedTopics$reliability <- reliability
 }
 else
 {
  alignedTopics$reliability <- reliability / K
  
  # evtl noch eine zufallskorrektur reinnehmen
  
 }
 
 return(alignedTopics)
}


reliability_cosine <- function(tm_fit1, tm_fit2, threshold = 0.8, topWordsToMatch = 50) {
  TM_Aligner(tm_fit1, tm_fit2, method = "cosine", thres = threshold, topWordsToMatch = topWordsToMatch, calculateBM = F)
}
reliability_lancichinetti <- function(tm_fit1, tm_fit2) {
  TM_Aligner(tm_fit1, tm_fit2, method = "doctopic", calculateBM = T)
}
reliability_kld <- function(tm_fit1, tm_fit2, threshold = 0.9, validWords = NULL) {
  TM_Aligner(tm_fit1, tm_fit2, method = "kullback", thres = threshold, calculateBM = F, validWords = validWords)
}
reliability_jsd <- function(tm_fit1, tm_fit2, threshold = 0.9, validWords = NULL) {
  TM_Aligner(tm_fit1, tm_fit2, method = "jensen", thres = threshold, calculateBM = F, validWords = validWords)
}


# align topic models
alignTopicModels <-
 function(dtm1, dtm2, ttm1, ttm2, K, topWordsToMatch = 50, similarityThreshold = 0.2) {
  c_topics1 <- toComparable(tm1$topics, topWordsToMatch)
  c_topics_p1 <- topicProbability(tm1$topics)
  c_topics2 <- toComparable(tm2$topics, topWordsToMatch)
  c_topics_p2 <- topicProbability(tm2$topics)
  alignedTopics <-
   TM_Aligner(
    c_topics1, c_topics2, similarityThreshold, c_topics_p1, c_topics_p2, topWordsToMatch
   )
  return(alignedTopics)
 }


printAlignedTopics <- function(alignedTopics, verbose = F) {
 if (verbose) {
  for (i in 1:length(alignedTopics$sharedTerms)) {
   if (length(alignedTopics$sharedTerms[[i]]) > 0) {
    cat(c(
     "___________________________________________________________", "\n"
    ))
    cat(c("Shared terms:", alignedTopics$sharedTerms[[i]], "\n"))
    cat(c(
     "Distance:", sprintf("%.4f", alignedTopics$distance[i]), "\n"
    ))
    cat(c(
     "Alignment:", alignedTopics$ids[i, 1], "TO", alignedTopics$ids[i, 2], "\n"
    ))
    cat(c(
     "Probabilities:", sprintf("%.4f", alignedTopics$probabilities[i, 1]), "TO:", sprintf("%.4f", alignedTopics$probabilities[i, 2]), "\n"
    ))
   }
  }
  cat(c(
   "===========================================================", "\n"
  ))
 }
 cat(c(
  "RELIABILITY:", sprintf("%.4f", alignedTopics$reliability), "\n"
 ))
}


RUN_AS_MAIN <- FALSE
if (RUN_AS_MAIN) {
 # Load Models
 tm1 <-
  get(
   load(file = "../online_climate_uk/model-1_K40_a0.002_e0.025_i10000rnd_d2015-12-18_16-28-51.RData")
  )
 tm2 <-
  get(
   load(file = "../online_climate_uk/model-1_K40_a0.002_e0.025_i10000fix_d2015-12-16_01-55-11.RData")
  )
 alignment <-
  alignTopicModels(tm1, tm2, topWordsToMatch = 100, similarityThreshold = 0.2)
 printAlignedTopics(alignment)
}
