# original code by A. Niekler & G. Wiedemann

# Berechnet topic coehrence nach (Mimno et al. 2011)
# - params: Dokument-Term-Matrix
# - LDA Modell
# - Anzahl an höchstwahrscheinlichen Termen zur Berechnung der Dokument-Kookkurrenz

tmca_topic_coherence <- function(DTM, theta, N = 25) {

  # Ensure matrix or Matrix-format (convert if slam)
  require(Matrix)
  require(slam)
  if (is.simple_triplet_matrix(DTM)) {
    DTM <- sparseMatrix(i=DTM$i, j=DTM$j, x=DTM$v, dims=c(DTM$nrow, DTM$ncol), dimnames = dimnames(DTM))
  }

  K <- nrow(theta)

  DTMBIN <- DTM > 0

  documentFrequency <- colSums(DTMBIN)
  names(documentFrequency) <- colnames(DTMBIN)

  require(lda)
  topNtermsPerTopic <- top.topic.words(theta, N, by.score=TRUE)
  allTopicModelTerms <- unique(as.vector(topNtermsPerTopic))

  DTMBIN <- DTMBIN[, allTopicModelTerms]
  DTMBINCooc <- t(DTMBIN) %*% DTMBIN
  DTMBINCooc <- t((DTMBINCooc + 1) / colSums(DTMBIN))
  DTMBINCooc <- log(DTMBINCooc)
  DTMBINCooc <- as.matrix(DTMBINCooc)

  coherence <- rep(0, K)
  pb <- txtProgressBar(max = K)
  for (topicIdx in 1:K) {
    setTxtProgressBar(pb, topicIdx)
    topWordsOfTopic <- topNtermsPerTopic[, topicIdx]
    coherence[topicIdx] <- 0
    for (m in 2:length(topWordsOfTopic)) {
      for (l in 1:(m-1)) {
        mTerm <- topWordsOfTopic[m]
        lTerm <- topWordsOfTopic[l]
        coherence[topicIdx] <- coherence[topicIdx] + DTMBINCooc[mTerm, lTerm]
      }
    }
  }
  close(pb)

  return(coherence)
}

